--[[
-- external functions
+ updateAvailableMounts()
+ initializeAddon()
+ summonMount(mountType, mountSpeed, namePattern)
+ companionList = listMounts(mountType, mountSpeed, namePattern)
- mountType, speed = getBestMount()
+ getChatCmd(input)

-- internal helpers
+ n = getTableSize(tbl)
+ companionIdList = spellID2companionID(spellId)
+ companionIdList = spellID2companionID(spellIdList)
- mountType, speed = getBestMountByLocation()
- mountType, speed = getBestMountBySkill()
+ mountIdList = queryMountDB(mountType=nil(=all), mountSpeed=nil(=all), namePattern=nil(=all))

-- chat functions
+sendChatMessage(message)
~showChatHelp()
]]--

-- -------------------------------------------------------------------------------------------
Mountaneer = {
	['availableMounts']	= {},
	['mountDB']			= {},
	['spell2companion']	= {},
	['frame']			= CreateFrame('frame');
}

function Mountaneer:initializeAddon()
	if not self.frame then
		self.frame = CreateFrame('frame');
	else
		self.frame:UnregisterEvent('PLAYER_LOGIN');	-- no need reloading this too often
	end;
	
	self.frame:RegisterEvent('COMPANION_UPDATE');
	self.frame:SetScript('OnEvent', 
		function(event, arg1, ...) 
			if event == 'COMPANION_UPDATE' and arg1 == 'MOUNT' then 
				Mountaneer:updateAvailableMounts() 
			end 
		end
	);
	
	-- do it manually the first time
	self:updateAvailableMounts();
	
	self:sendChatMessage('succesfully loaded.');
end;

function Mountaneer:updateAvailableMounts()
	-- clean the cache first, so we don't get any stale data (eg. due to a skill-increase. Some mounts change speed based on skill)
	self['availableMounts']	= {};
	self['spell2companion']	= {};
	
	-- no sense updating the list if there ARE no mounts.
	if GetNumCompanions('MOUNT') == 0 then
		return;
	end;
	
	-- make a list of all spellID's of the mounts we own
	for i=1,GetNumCompanions('MOUNT') do
		_, _, spellID = GetCompanionInfo('MOUNT', i);
		self['spell2companion'][spellID] = i;
	end;
	
	-- insert mounts found into the cache in the proper category
	for catName, content in pairs(self['mountDB']) do
		self['availableMounts'][catName] = {};
		
		for _, mount in ipairs(content) do
			if self['spell2companion'][mount] then
				self['availableMounts'][catName][#(self['availableMounts'][catName])+1] = mount;
			end;
		end;
	end;
end;

function Mountaneer:sendChatMessage(message, prefix)
	if prefix == nil then
		prefix = '|cff00ff00Mountaneer:|r ';
	end;
	
	DEFAULT_CHAT_FRAME:AddMessage(prefix..message);
end;

function Mountaneer:showChatHelp()
	self:sendChatMessage('Help');
	self:sendChatMessage('|rUsage: /|cffffff00mount|r [|cffffff00actionType|r] [|cffffff00mountType|r] [|cffffff00mountSpeed|r] [|cffffff00name|r]', '');
	
	self:sendChatMessage('|r[|cffffff00actionType|r]:',' ');
	self:sendChatMessage('|r|cffffff00help|r - Will display a list of possible options, similair to this page. All options after help are ignored.', '   ');
	self:sendChatMessage('|r|cffffff00max|r - This is a special option. It will cause Mountaneer to auto-summon the best possible mount, taking into account your riding skill and the location. Thus, it will summon a (random) land mount in Darnassus, and a flying mount on Krassus\' landing. All options after max are ignored.', '   ');
	self:sendChatMessage('|r|cffffff00list|r - This option will show a list of all mounts matching your query, without auto-summoning a random one from the list.', '   ');    
	self:sendChatMessage('|r|cffffff00summon|r - This option will try to summon a (random) mount matching the rest of the query.', '   ');
	self:sendChatMessage('|r[|cffffff00none|r] - When no option is provided for actionType, Mountaneer will auto-summon a random mount matching the rest of the query.', '   ');    
	
	self:sendChatMessage('|r[|cffffff00mountType|r]:',' ');
	self:sendChatMessage('|r|cffffff00special|r - This signifies a special mount which does not increase the runningspeed of your character in any way; currently the only such mount available is the riding turtle.', '   ');
	self:sendChatMessage('|r|cffffff00swimming|r - This signifies a special mount which increases swimming speed. Currently the only mount to do this is the Sea turtle.', '   ');
	self:sendChatMessage('|r|cffffff00land|r - This signifies a mount which increases speed on land only. This does not include flying mounts, even though they do increase running speed on land aswell. You\'re better off using a fast land mount anyway.', '   ');
	self:sendChatMessage('|r|cffffff00flying|r - This signifies a flying mount, and can currently only be used in Northrend and The Outlands.', '   ');
	self:sendChatMessage('|r[|cffffff00none|r] - When no option is provided for mountType, Mountaneer will try to pick a mount from any of the above mentioned categories.', '   ');
	
	self:sendChatMessage('|r[|cffffff00mountSpeed|r]:',' ');
	self:sendChatMessage('|r|cffffff00slow|r - This signifies you wish to summon a slow mount, meaning a +60% land mount or a +150% flying mount, based on the mountType option above.', '   ');
	self:sendChatMessage('|r|cffffff00rare|r - Alias for |r|cffffff00slow|r.', '   ');
	self:sendChatMessage('|r|cffffff00fast|r - This signifies you wish to summon a fast mount, meaning a +100% land mount or a +280% flying mount, based on the mountType option above. Although an extremely fast (+310%) mount is also fast, it does not fall in this category, but rather falls in the extreme category.', '   ');
	self:sendChatMessage('|r|cffffff00epic|r - Alias for |r|cffffff00fast|r.', '   ');
	self:sendChatMessage('|r|cffffff00extreme|r - This signifies you wish to summon an extremely fast mount, meaning a +310% flying mount.', '   ');
	
	self:sendChatMessage('|r[|cffffff00name|r]:',' ');
	self:sendChatMessage('|r|cffffff00random|r - This signifies you wish to pick a random mount from the before-mentioned categories.', '   ');
	self:sendChatMessage('|r[|cffffff00name|r] - You may add a (partial) (case-insensitive) name to try to summon a particular (group of) mount(s). ', '   ');
	
	self:sendChatMessage('|r|cff00ff00Example:|r', ' ');
	self:sendChatMessage('|r/|cffffff00mount summon land fast random|r - This will summon a random fast land-mount', '  ');
	self:sendChatMessage('|r/|cffffff00mount max|r - This will summon the best mount you have and can fly here, so in order: a +310% flying mount, +280% flying, +150% flying, +100% land, +60% land, riding turtle.', '  ');
end;

function Mountaneer:summonMount(mountType, mountSpeed, namePattern)
	local mountList = self:queryMountDB(mountType, mountSpeed, namePattern);
	local n = self:getTableSize(mountList);
	
	if n > 0 then
		local result = random(1, n);
		local companionId = self:spellID2companionID(mountList[result]);
		
		local link = GetSpellLink(mountList[result]);
		
		self:sendChatMessage('Summoning '..link);
		CallCompanion('MOUNT', companionId);
	else
		self:sendChatMessage('No mounts found matching your query.');
	end;
end;

function Mountaneer:listMounts(mountType, mountSpeed, namePattern)
	-- list of companionID's
	local mountList = self:queryMountDB(mountType, mountSpeed, namePattern);
	local n = self:getTableSize(mountList);
	
	self:sendChatMessage(n..' mounts matched your query:');
	if n > 0 then
		for _, spellId in pairs(mountList) do			
			self:sendChatMessage(GetSpellLink(spellId), '  ');
		end;
	end;
end;

function Mountaneer:queryMountDB(mountType, mountSpeed, namePattern)
	local returnTable = {};
 	local cats = {};
	
	if mountType == 'any' then
		-- add categories, limited by speed
		cats[#cats+1] = 'special';
		cats[#cats+1] = 'swimming';

		if mountSpeed == 'slow' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-slow';
			cats[#cats+1] = 'flying-slow';
		end;
		if mountSpeed == 'fast' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-fast';
			cats[#cats+1] = 'flying-fast';
		end;
		if mountSpeed == 'extreme' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-extreme';
		end;
		
	elseif mountType == 'land' then
		if mountSpeed == 'slow' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-slow';
		end;
		if mountSpeed == 'fast' or mountSpeed == 'any' then
			cats[#cats+1] = 'land-fast';
		end;
	elseif mountType == 'flying' then
		if mountSpeed == 'slow' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-slow';
		end;
		if mountSpeed == 'fast' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-fast';
		end;
		if mountSpeed == 'extreme' or mountSpeed == 'any' then
			cats[#cats+1] = 'flying-extreme';
		end;
	elseif mountType == 'swimming' then
		cats[#cats+1] = 'swimming';
	elseif mountType == 'special' then
		cats[#cats+1] = 'special';
	end;
	
	-- make a complete list with all mounts from the type/speed combo's
	for i=1,#cats do
		for j=1,#(self.availableMounts[cats[i]]) do
			returnTable[#returnTable+1] = self.availableMounts[cats[i]][j];
		end;
	end;
	
	-- filter the list based on name
	if namePattern == 'random' then
		return returnTable;	-- special case, return all
	else
		local finalReturn = {};
		namePattern = string.lower(namePattern);	-- make case-insensitive
		
		for i=1,#returnTable do
			local spellId = returnTable[i];
			local mountNameCS = GetSpellInfo(spellId);
			local mountName = string.lower(mountNameCS);	-- case-insensitive
			if string.match(mountName, namePattern) then
				finalReturn[#finalReturn+1] = spellId;
			end;
		end;
		
		return finalReturn;
	end;
end;

function Mountaneer:spellID2companionID(input)
	if type(input) == 'table' then
		local returnTbl = {};
		
		for i=1,#input do
			returnTbl[#returnTbl+1] = self['spell2companion'][input[i]];
		end;
		
		return returnTbl;
	else
		return self['spell2companion'][input];
	end;
end;

function Mountaneer:getTableSize(tbl)
	local n = 0;
	
	for _ in pairs(tbl) do
		n = n+1;
	end;
	
	return n;
end;

function Mountaneer:getChatCmd(input)
	local parts = {strsplit(' ', input)};

	--[[
		[actionType] [mountType] [mountSpeed] [namePattern]
	]]--
	
	local actionType, mountType, mountSpeed, namePattern = nil, nil, nil, nil;
	local depth = 1;
	local querySize = self:getTableSize(parts);
	local doneParsing = false;
	
	-- figure out what is what in the string
	if querySize == 0 then
		actionType = 'help';
	else
		-- [actionType]
		if parts[depth] == 'help' or parts[depth] == 'max' or parts[depth] == 'init' then
			actionType = parts[depth];
			doneParsing = true;
		elseif parts[depth] == 'list' or parts[depth] == 'summon' then
			actionType = parts[depth];
			depth = depth+1;
		else
			actionType = 'summon';	-- default value
		end;
		
		-- [mountType]
		if(not doneParsing and querySize >= depth) then
			if parts[depth] == 'land' or parts[depth] == 'flying' or parts[depth] == 'swimming' or parts[depth] == 'special' then
				mountType = parts[depth];
				depth = depth+1;
			else
				mountType = 'any';	-- default value
			end;
		end;

		-- [mountSpeed]
		if(not doneParsing and querySize >= depth) then
			if parts[depth] == 'slow' or parts[depth] == 'rare' then
				mountSpeed = 'slow';
				depth = depth+1;
			elseif parts[depth] == 'fast' or parts[depth] == 'epic' then
				mountSpeed = 'fast';
				depth = depth+1;
			elseif parts[depth] == 'extreme' then
				mountSpeed = 'extreme';
				depth = depth+1;
			elseif parts[depth] == 'any' then
				mountSpeed = 'any';
				depth = depth+1;
			else
				mountSpeed = 'any';	-- default value
			end;
		end;

		-- [namePattern]
		if(not doneParsing and querySize >= depth) then
			local nameParts = '';
			for i=depth,querySize do
				if nameParts == '' then
					nameParts = parts[i];
				else
					nameParts = nameParts..' '..parts[i];
				end;
			end;
			
			namePattern = nameParts;
		else
			namePattern = 'random';
		end;
	end;
	
	
	-- perform actual action
	if actionType == 'help' then
		self:showChatHelp();
	
	elseif actionType == 'init' then
		self:initializeAddon();
	
	elseif actionType == 'max' then
		self:sendChatMessage('/mount max has not been implemented yet');
		
		--mountType, mountSpeed = self:getBestMount();
		--self:summonMount(mountType, mountSpeed, nil);
	
	elseif actionType == 'list' then
		self:listMounts(mountType, mountSpeed, namePattern);
	
	elseif actionType == 'summon' then
		self:summonMount(mountType, mountSpeed, namePattern);
	end;
end;

SLASH_MOUNTANEER1 = '/mount';
SlashCmdList['MOUNTANEER'] = function(input) Mountaneer:getChatCmd(input) end;

Mountaneer.frame:RegisterEvent('PLAYER_LOGIN');
Mountaneer.frame:SetScript('OnEvent', function() Mountaneer:initializeAddon() end);